@@ -23,6 +23,7 @@
 #include "telepathy-glib/capabilities-internal.h"
 
 #include <telepathy-glib/dbus.h>
+#include <telepathy-glib/dbus-internal.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/gtypes.h>
 #include <telepathy-glib/interfaces.h>
@@ -30,6 +31,7 @@
 
 #define DEBUG_FLAG TP_DEBUG_CONNECTION
 #include "telepathy-glib/debug-internal.h"
+#include "telepathy-glib/variant-util-internal.h"
 
 /**
  * SECTION:capabilities
@@ -66,12 +68,14 @@ G_DEFINE_TYPE (TpCapabilities, tp_capabilities, G_TYPE_OBJECT)
 enum {
     PROP_CHANNEL_CLASSES = 1,
     PROP_CONTACT_SPECIFIC,
+    PROP_CHANNEL_CLASSES_VARIANT,
     N_PROPS
 };
 
 struct _TpCapabilitiesPrivate {
     GPtrArray *classes;
     gboolean contact_specific;
+    GVariant *classes_variant;
 };
 
 /**
@@ -136,6 +140,8 @@ tp_capabilities_dispose (GObject *object)
       self->priv->classes = NULL;
     }
 
+  tp_clear_pointer (&self->priv->classes_variant, g_variant_unref);
+
   ((GObjectClass *) tp_capabilities_parent_class)->dispose (object);
 }
 
@@ -157,6 +163,10 @@ tp_capabilities_get_property (GObject *object,
       g_value_set_boolean (value, self->priv->contact_specific);
       break;
 
+    case PROP_CHANNEL_CLASSES_VARIANT:
+      g_value_set_variant (value, self->priv->classes_variant);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -175,6 +185,9 @@ tp_capabilities_set_property (GObject *object,
     {
     case PROP_CHANNEL_CLASSES:
       self->priv->classes = g_value_dup_boxed (value);
+      self->priv->classes_variant = _tp_boxed_to_variant (
+          TP_ARRAY_TYPE_REQUESTABLE_CHANNEL_CLASS_LIST, "a(a{sv}as)",
+          self->priv->classes);
       break;
 
     case PROP_CONTACT_SPECIFIC:
@@ -236,6 +249,32 @@ tp_capabilities_class_init (TpCapabilitiesClass *klass)
       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONTACT_SPECIFIC,
       param_spec);
+
+  /**
+   * TpCapabilities:channel-classes-variant:
+   *
+   * The underlying data structure used by Telepathy to represent the
+   * requests that can succeed.
+   *
+   * This can be used by advanced clients to determine whether an unusually
+   * complex request would succeed. See the Telepathy D-Bus API Specification
+   * for details of how to interpret the returned #GVariant of type
+   * a(a{sv}as).
+   *
+   * The higher-level methods like
+   * tp_capabilities_supports_text_chats() are likely to be more useful to
+   * the majority of clients.
+   *
+   * Since: 0.19.0
+   */
+  param_spec = g_param_spec_variant ("channel-classes-variant",
+      "GVariant of type a(a{sv}as)",
+      "The channel classes supported",
+      G_VARIANT_TYPE ("a(a{sv}as)"),
+      NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_CHANNEL_CLASSES_VARIANT,
+      param_spec);
 }
 
 static void
@@ -312,7 +351,9 @@ supports_simple_channel (TpCapabilities *self,
  * @self: a #TpCapabilities object
  *
  * Return whether private text channels can be established by providing
- * a contact identifier.
+ * a contact identifier, for instance by calling
+ * tp_account_channel_request_new_text() followed by
+ * tp_account_channel_request_set_target_contact().
  *
  * If the protocol is such that text chats can be established, but only via a
  * more elaborate D-Bus API than normal (because more information is needed),
@@ -337,7 +378,9 @@ tp_capabilities_supports_text_chats (TpCapabilities *self)
  *
  * If the #TpCapabilities:contact-specific property is %FALSE, this function
  * checks if named text chatrooms can be joined by providing a chatroom
- * identifier.
+ * identifier, for instance by calling
+ * tp_account_channel_request_new_text() followed by
+ * tp_account_channel_request_set_target_id() with %TP_HANDLE_TYPE_ROOM.
  *
  * If the #TpCapabilities:contact-specific property is %TRUE, this function
  * checks if the contact associated with this #TpCapabilities can be invited
@@ -360,6 +403,84 @@ tp_capabilities_supports_text_chatrooms (TpCapabilities *self)
       TP_HANDLE_TYPE_ROOM);
 }
 
+/**
+ * tp_capabilities_supports_sms:
+ * @self: a #TpCapabilities object
+ *
+ * If the #TpCapabilities:contact-specific property is %FALSE, this function
+ * checks if SMS text channels can be requested with the connection associated
+ * with this #TpCapabilities.
+ *
+ * If the #TpCapabilities:contact-specific property is %TRUE, this function
+ * checks if the contact associated with this #TpCapabilities supports
+ * SMS text channels.
+ *
+ * Returns: %TRUE if a channel request containing Text as ChannelType,
+ * HandleTypeContact as TargetHandleType, a channel identifier and
+ * #TP_PROP_CHANNEL_INTERFACE_SMS_SMS_CHANNEL set to %TRUE can be
+ * expected to work, %FALSE otherwise.
+ *
+ * Since: 0.19.0
+ */
+gboolean
+tp_capabilities_supports_sms (TpCapabilities *self)
+{
+  guint i;
+
+  g_return_val_if_fail (TP_IS_CAPABILITIES (self), FALSE);
+
+  for (i = 0; i < self->priv->classes->len; i++)
+    {
+      GValueArray *arr = g_ptr_array_index (self->priv->classes, i);
+      GHashTable *fixed;
+      const gchar * const *allowed;
+      const gchar *chan_type;
+      TpHandleType handle_type;
+      gboolean valid;
+      guint nb_fixed_props;
+
+      fixed =  g_value_get_boxed (g_value_array_get_nth (arr, 0));
+      allowed = g_value_get_boxed (g_value_array_get_nth (arr, 1));
+
+      handle_type = tp_asv_get_uint32 (fixed,
+          TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, &valid);
+
+      if (!valid)
+        continue;
+
+      if (handle_type != TP_HANDLE_TYPE_CONTACT)
+        continue;
+
+      chan_type = tp_asv_get_string (fixed, TP_PROP_CHANNEL_CHANNEL_TYPE);
+
+      if (tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_TEXT))
+        continue;
+
+      /* SMSChannel be either in fixed or allowed properties */
+      if (tp_asv_get_boolean (fixed, TP_PROP_CHANNEL_INTERFACE_SMS_SMS_CHANNEL,
+            NULL))
+        {
+          /* In fixed, succeed if there is no more fixed properties required */
+          nb_fixed_props = 3;
+        }
+      else
+        {
+          /* Not in fixed; check allowed */
+          if (!tp_strv_contains (allowed,
+                TP_PROP_CHANNEL_INTERFACE_SMS_SMS_CHANNEL))
+            continue;
+
+          nb_fixed_props = 2;
+        }
+
+      if (g_hash_table_size (fixed) == nb_fixed_props)
+        return TRUE;
+    }
+
+  return FALSE;
+
+}
+
 static gboolean
 supports_call_full (TpCapabilities *self,
     TpHandleType expected_handle_type,
@@ -378,6 +499,7 @@ supports_call_full (TpCapabilities *self,
       const gchar *chan_type;
       TpHandleType handle_type;
       gboolean valid;
+      guint nb_fixed_props = 2;
 
       tp_value_array_unpack (arr, 2,
           &fixed_prop,
@@ -392,22 +514,39 @@ supports_call_full (TpCapabilities *self,
       if (!valid || handle_type != expected_handle_type)
         continue;
 
-      if (expected_initial_audio &&
-          !tp_asv_get_boolean (fixed_prop,
-              TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO, NULL) &&
-          !tp_strv_contains (allowed_prop,
-              TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO))
-        continue;
+      if (expected_initial_audio)
+        {
+          /* We want audio, INITIAL_AUDIO must be in either fixed or allowed */
+          if (tp_asv_get_boolean (fixed_prop,
+                  TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO, NULL))
+            {
+              nb_fixed_props++;
+            }
+          else if (!tp_strv_contains (allowed_prop,
+                  TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO))
+            {
+              continue;
+            }
+        }
 
-      if (expected_initial_video &&
-          !tp_asv_get_boolean (fixed_prop,
-              TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO, NULL) &&
-          !tp_strv_contains (allowed_prop,
-              TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO))
-        continue;
+      if (expected_initial_video)
+        {
+          /* We want video, INITIAL_VIDEO must be in either fixed or allowed */
+          if (tp_asv_get_boolean (fixed_prop,
+                  TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO, NULL))
+            {
+              nb_fixed_props++;
+            }
+          else if (!tp_strv_contains (allowed_prop,
+                  TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO))
+            {
+              continue;
+            }
+        }
 
       /* We found the right class */
-      return TRUE;
+      if (g_hash_table_size (fixed_prop) == nb_fixed_props)
+        return TRUE;
     }
 
   return FALSE;
@@ -420,7 +559,13 @@ supports_call_full (TpCapabilities *self,
  *  private, #TP_HANDLE_TYPE_ROOM or #TP_HANDLE_TYPE_NONE for conference
  *  (depending on the protocol)
  *
- * Return whether audio call can be established.
+ * Return whether audio calls can be established, for instance by calling
+ * tp_account_channel_request_new_audio_call(), followed by
+ * tp_account_channel_request_set_target_id() with @handle_type.
+ *
+ * To check whether requests using
+ * tp_account_channel_request_set_target_contact() would work, set
+ * @handle_type to %TP_HANDLE_TYPE_CONTACT.
  *
  * Returns: %TRUE if a channel request containing Call as ChannelType,
  * @handle_type as TargetHandleType, a True value for InitialAudio and an
@@ -442,7 +587,13 @@ tp_capabilities_supports_audio_call (TpCapabilities *self,
  *  private, #TP_HANDLE_TYPE_ROOM or #TP_HANDLE_TYPE_NONE for conference
  *  (depending on the protocol)
  *
- * Return whether audio/video call can be established.
+ * Return whether audio/video calls can be established, for instance by calling
+ * tp_account_channel_request_new_audio_video_call(), followed by
+ * tp_account_channel_request_set_target_id() with @handle_type.
+ *
+ * To check whether requests using
+ * tp_account_channel_request_set_target_contact() would work, set
+ * @handle_type to %TP_HANDLE_TYPE_CONTACT.
  *
  * Returns: %TRUE if a channel request containing Call as ChannelType,
  * @handle_type as TargetHandleType, a True value for
@@ -459,6 +610,95 @@ tp_capabilities_supports_audio_video_call (TpCapabilities *self,
   return supports_call_full (self, handle_type, TRUE, TRUE);
 }
 
+typedef enum {
+    FT_CAP_FLAGS_NONE = 0,
+    FT_CAP_FLAG_URI = (1<<0),
+    FT_CAP_FLAG_OFFSET = (1<<1),
+    FT_CAP_FLAG_DATE = (1<<2),
+    FT_CAP_FLAG_DESCRIPTION = (1<<3)
+} FTCapFlags;
+
+static gboolean
+supports_file_transfer (TpCapabilities *self,
+    FTCapFlags flags)
+{
+  guint i;
+
+  g_return_val_if_fail (TP_IS_CAPABILITIES (self), FALSE);
+
+  for (i = 0; i < self->priv->classes->len; i++)
+    {
+      GValueArray *arr = g_ptr_array_index (self->priv->classes, i);
+      GHashTable *fixed;
+      const gchar *chan_type;
+      TpHandleType handle_type;
+      gboolean valid;
+      guint n_fixed = 2;
+      const gchar * const *allowed;
+
+      tp_value_array_unpack (arr, 2, &fixed, &allowed);
+
+      chan_type = tp_asv_get_string (fixed, TP_PROP_CHANNEL_CHANNEL_TYPE);
+
+      if (tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER))
+        continue;
+
+      handle_type = tp_asv_get_uint32 (fixed,
+          TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, &valid);
+
+      if (!valid)
+        continue;
+
+      if (handle_type != TP_HANDLE_TYPE_CONTACT)
+        continue;
+
+      /* ContentType, Filename, Size are mandatory. In principle we could check
+       * that the CM allows them, but not allowing them would be ridiculous,
+       * so we don't.
+       */
+
+      if ((flags & FT_CAP_FLAG_DESCRIPTION) != 0)
+        {
+          /* Description makes no sense as a fixed property so we assume
+           * the CM won't be ridiculous */
+          if (!tp_strv_contains (allowed,
+                TP_PROP_CHANNEL_TYPE_FILE_TRANSFER_DESCRIPTION))
+            continue;
+        }
+
+      if ((flags & FT_CAP_FLAG_DATE) != 0)
+        {
+          /* makes no sense as a fixed property */
+          if (!tp_strv_contains (allowed,
+                TP_PROP_CHANNEL_TYPE_FILE_TRANSFER_DATE))
+            continue;
+        }
+
+      if ((flags & FT_CAP_FLAG_URI) != 0)
+        {
+          /* makes no sense as a fixed property */
+          if (!tp_strv_contains (allowed,
+                TP_PROP_CHANNEL_TYPE_FILE_TRANSFER_URI))
+            continue;
+        }
+
+      if ((flags & FT_CAP_FLAG_OFFSET) != 0)
+        {
+          /* makes no sense as a fixed property */
+          if (!tp_strv_contains (allowed,
+                TP_PROP_CHANNEL_TYPE_FILE_TRANSFER_INITIAL_OFFSET))
+            continue;
+        }
+
+      if (n_fixed != tp_asv_size (fixed))
+        continue;
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 /**
  * tp_capabilities_supports_file_transfer:
  * @self: a #TpCapabilities object
@@ -475,8 +715,81 @@ tp_capabilities_supports_audio_video_call (TpCapabilities *self,
 gboolean
 tp_capabilities_supports_file_transfer (TpCapabilities *self)
 {
-  return supports_simple_channel (self, TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER,
-      TP_HANDLE_TYPE_CONTACT);
+  return supports_file_transfer (self, FT_CAP_FLAGS_NONE);
+}
+
+/**
+ * tp_capabilities_supports_file_transfer_uri:
+ * @self: a #TpCapabilities object
+ *
+ * <!-- -->
+ *
+ * Returns: %TRUE if requests as described for
+ *  tp_capabilities_supports_file_transfer() can also specify the outgoing
+ *  file's URI
+ *
+ * Since: 0.19.0
+ */
+gboolean
+tp_capabilities_supports_file_transfer_uri (TpCapabilities *self)
+{
+  return supports_file_transfer (self, FT_CAP_FLAG_URI);
+}
+
+/**
+ * tp_capabilities_supports_file_transfer_description:
+ * @self: a #TpCapabilities object
+ *
+ * <!-- -->
+ *
+ * Returns: %TRUE if requests as described for
+ *  tp_capabilities_supports_file_transfer() can also specify the outgoing
+ *  file's description
+ *
+ * Since: 0.19.0
+ */
+gboolean
+tp_capabilities_supports_file_transfer_description (TpCapabilities *self)
+{
+  return supports_file_transfer (self, FT_CAP_FLAG_DESCRIPTION);
+}
+
+/**
+ * tp_capabilities_supports_file_transfer_initial_offset:
+ * @self: a #TpCapabilities object
+ *
+ * Return whether an initial offset other than 0 can be specified on
+ * outgoing file transfers. This can be used to resume partial transfers,
+ * by omitting the part that has already been sent.
+ *
+ * Returns: %TRUE if requests as described for
+ *  tp_capabilities_supports_file_transfer() can also specify an
+ *  initial offset greater than 0
+ *
+ * Since: 0.19.0
+ */
+gboolean
+tp_capabilities_supports_file_transfer_initial_offset (TpCapabilities *self)
+{
+  return supports_file_transfer (self, FT_CAP_FLAG_OFFSET);
+}
+
+/**
+ * tp_capabilities_supports_file_transfer_timestamp:
+ * @self: a #TpCapabilities object
+ *
+ * <!-- -->
+ *
+ * Returns: %TRUE if requests as described for
+ *  tp_capabilities_supports_file_transfer() can also specify the outgoing
+ *  file's timestamp
+ *
+ * Since: 0.19.0
+ */
+gboolean
+tp_capabilities_supports_file_transfer_timestamp (TpCapabilities *self)
+{
+  return supports_file_transfer (self, FT_CAP_FLAG_DATE);
 }
 
 static gboolean
@@ -499,35 +812,31 @@ tp_capabilities_supports_tubes_common (TpCapabilities *self,
       const gchar *chan_type;
       TpHandleType handle_type;
       gboolean valid;
-      const gchar *service;
+      guint nb_fixed_props = 2;
 
       fixed =  g_value_get_boxed (g_value_array_get_nth (arr, 0));
 
       chan_type = tp_asv_get_string (fixed, TP_PROP_CHANNEL_CHANNEL_TYPE);
-
       if (tp_strdiff (chan_type, expected_channel_type))
         continue;
 
       handle_type = tp_asv_get_uint32 (fixed,
           TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, &valid);
-
-      if (!valid)
+      if (!valid || handle_type != expected_handle_type)
         continue;
 
-      if (handle_type != expected_handle_type)
-        continue;
+      if (expected_service != NULL && self->priv->contact_specific)
+        {
+          const gchar *service;
 
-      if (expected_service == NULL || !self->priv->contact_specific)
-        /* No need to check the service */
+          nb_fixed_props++;
+          service = tp_asv_get_string (fixed, service_prop);
+          if (tp_strdiff (service, expected_service))
+            continue;
+        }
+
+      if (g_hash_table_size (fixed) == nb_fixed_props)
         return TRUE;
-
-      service = tp_asv_get_string (fixed, service_prop);
-
-      if (tp_strdiff (service, expected_service))
-        continue;
-
-      /* We found the right service */
-      return TRUE;
     }
 
   return FALSE;
@@ -642,11 +951,13 @@ tp_capabilities_supports_contact_search (TpCapabilities *self,
 
       tp_value_array_unpack (arr, 2, &fixed, &allowed_properties);
 
+      /* ContactSearch channel should have ChannelType and TargetHandleType=NONE
+       * but CM implementations are wrong and omitted TargetHandleType,
+       * so it's set in stone now.  */
       if (g_hash_table_size (fixed) != 1)
         continue;
 
       chan_type = tp_asv_get_string (fixed, TP_PROP_CHANNEL_CHANNEL_TYPE);
-
       if (tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_CONTACT_SEARCH))
         continue;
 
@@ -736,14 +1047,23 @@ tp_capabilities_supports_room_list (TpCapabilities *self,
       GHashTable *fixed;
       const gchar *chan_type;
       const gchar **allowed_properties;
+      TpHandleType handle_type;
+      gboolean valid;
 
       tp_value_array_unpack (arr, 2, &fixed, &allowed_properties);
 
-      chan_type = tp_asv_get_string (fixed, TP_PROP_CHANNEL_CHANNEL_TYPE);
+      if (g_hash_table_size (fixed) != 2)
+        continue;
 
+      chan_type = tp_asv_get_string (fixed, TP_PROP_CHANNEL_CHANNEL_TYPE);
       if (tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_ROOM_LIST))
         continue;
 
+      handle_type = tp_asv_get_uint32 (fixed,
+          TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, &valid);
+      if (!valid || handle_type != TP_HANDLE_TYPE_NONE)
+        continue;
+
       result = TRUE;
 
       server = tp_strv_contains (allowed_properties,
@@ -756,3 +1076,20 @@ tp_capabilities_supports_room_list (TpCapabilities *self,
 
   return result;
 }
+
+/**
+ * tp_capabilities_dup_channel_classes_variant:
+ * @self: a #TpCapabilities
+ *
+ * Return the #TpCapabilities:channel-classes-variant property
+ *
+ * Returns: (transfer full): the value of the
+ * #TpCapabilities:channel-classes-variant property
+ *
+ * Since: 0.19.0
+ */
+GVariant *
+tp_capabilities_dup_channel_classes_variant (TpCapabilities *self)
+{
+  return g_variant_ref (self->priv->classes_variant);
+}
