@@ -18,10 +18,15 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#if defined (TP_DISABLE_SINGLE_INCLUDE) && !defined (_TP_IN_META_HEADER) && !defined (_TP_COMPILATION)
+#error "Only <telepathy-glib/telepathy-glib.h> and <telepathy-glib/telepathy-glib-dbus.h> can be included directly."
+#endif
+
 #ifndef __TP_DBUS_TUBE_CHANNEL_H__
 #define __TP_DBUS_TUBE_CHANNEL_H__
 
 #include <telepathy-glib/channel.h>
+#include <telepathy-glib/defs.h>
 
 G_BEGIN_DECLS
 
@@ -52,31 +57,42 @@ struct _TpDBusTubeChannelClass
 
 #define TP_DBUS_TUBE_CHANNEL_FEATURE_CORE \
   tp_dbus_tube_channel_feature_quark_core ()
+_TP_AVAILABLE_IN_0_18
 GQuark tp_dbus_tube_channel_feature_quark_core (void) G_GNUC_CONST;
 
+_TP_AVAILABLE_IN_0_18
 GType tp_dbus_tube_channel_get_type (void);
 
+_TP_AVAILABLE_IN_0_18
 const gchar * tp_dbus_tube_channel_get_service_name (TpDBusTubeChannel *self);
 
+_TP_AVAILABLE_IN_0_18
 GHashTable * tp_dbus_tube_channel_get_parameters (TpDBusTubeChannel *self);
 
+_TP_AVAILABLE_IN_0_20
+GVariant * tp_dbus_tube_channel_dup_parameters_vardict (TpDBusTubeChannel *self);
+
 /* Outgoing tube methods */
 
+_TP_AVAILABLE_IN_0_18
 void tp_dbus_tube_channel_offer_async (TpDBusTubeChannel *self,
     GHashTable *params,
     GAsyncReadyCallback callback,
     gpointer user_data);
 
+_TP_AVAILABLE_IN_0_18
 GDBusConnection * tp_dbus_tube_channel_offer_finish (TpDBusTubeChannel *self,
     GAsyncResult *result,
     GError **error) G_GNUC_WARN_UNUSED_RESULT;
 
 /* Incoming tube methods */
 
+_TP_AVAILABLE_IN_0_18
 void tp_dbus_tube_channel_accept_async (TpDBusTubeChannel *self,
     GAsyncReadyCallback callback,
     gpointer user_data);
 
+_TP_AVAILABLE_IN_0_18
 GDBusConnection * tp_dbus_tube_channel_accept_finish (TpDBusTubeChannel *self,
     GAsyncResult *result,
     GError **error) G_GNUC_WARN_UNUSED_RESULT;
