@@ -109,6 +109,7 @@
 
 #define DEBUG_FLAG TP_DEBUG_CHANNEL
 #include "telepathy-glib/automatic-client-factory-internal.h"
+#include "telepathy-glib/channel-internal.h"
 #include "telepathy-glib/debug-internal.h"
 
 #include <stdio.h>
@@ -184,7 +185,7 @@ operation_failed (TpFileTransferChannel *self,
     GError *error)
 {
   g_simple_async_result_take_error (self->priv->result, error);
-  g_simple_async_result_complete (self->priv->result);
+  g_simple_async_result_complete_in_idle (self->priv->result);
   tp_clear_object (&self->priv->result);
 }
 
@@ -441,7 +442,7 @@ tp_file_transfer_channel_prepare_core_cb (TpProxy *proxy,
     self->priv->file = g_file_new_for_uri (uri);
 
 out:
-  g_simple_async_result_complete (result);
+  g_simple_async_result_complete_in_idle (result);
 }
 
 static void
@@ -530,7 +531,7 @@ tp_file_transfer_channel_constructed (GObject *obj)
 
   G_OBJECT_CLASS (tp_file_transfer_channel_parent_class)->constructed (obj);
 
-  properties = tp_channel_borrow_immutable_properties (TP_CHANNEL (self));
+  properties = _tp_channel_get_immutable_properties (TP_CHANNEL (self));
 
   self->priv->mime_type = tp_asv_get_string (properties,
     TP_PROP_CHANNEL_TYPE_FILE_TRANSFER_CONTENT_TYPE);
@@ -880,7 +881,7 @@ tp_file_transfer_channel_class_init (TpFileTransferChannelClass *klass)
   param_spec = g_param_spec_uint ("state",
       "State",
       "The TpFileTransferState of the channel",
-      0, NUM_TP_FILE_TRANSFER_STATES, TP_FILE_TRANSFER_STATE_NONE,
+      0, TP_NUM_FILE_TRANSFER_STATES, TP_FILE_TRANSFER_STATE_NONE,
       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_STATE,
       param_spec);
@@ -1058,6 +1059,7 @@ tp_file_transfer_channel_get_feature_quark_core (void)
  * Returns: (transfer full): a newly created #TpFileTransferChannel
  *
  * Since: 0.15.5
+ * Deprecated: Use tp_simple_client_factory_ensure_channel() instead.
  */
 TpFileTransferChannel *
 tp_file_transfer_channel_new (TpConnection *conn,
@@ -1168,7 +1170,7 @@ accept_or_provide_file_cb (TpChannel *proxy,
       start_transfer (self);
     }
 
-  g_simple_async_result_complete (self->priv->result);
+  g_simple_async_result_complete_in_idle (self->priv->result);
 }
 
 static gboolean
@@ -1328,7 +1330,7 @@ tp_file_transfer_channel_accept_file_async (TpFileTransferChannel *self,
   if (self->priv->access_control_param != NULL)
     {
       g_simple_async_report_error_in_idle (G_OBJECT (self), callback,
-          user_data, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+          user_data, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "Can't accept already accepted transfer");
 
       return;
@@ -1337,7 +1339,7 @@ tp_file_transfer_channel_accept_file_async (TpFileTransferChannel *self,
   if (self->priv->state != TP_FILE_TRANSFER_STATE_PENDING)
     {
       g_simple_async_report_error_in_idle (G_OBJECT (self), callback,
-          user_data, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+          user_data, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "Can't accept a transfer that isn't pending");
 
       return;
@@ -1346,7 +1348,7 @@ tp_file_transfer_channel_accept_file_async (TpFileTransferChannel *self,
   if (tp_channel_get_requested (TP_CHANNEL (self)))
     {
       g_simple_async_report_error_in_idle (G_OBJECT (self), callback,
-          user_data, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+          user_data, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "Can't accept outgoing transfer");
 
       return;
@@ -1449,7 +1451,7 @@ tp_file_transfer_channel_provide_file_async (TpFileTransferChannel *self,
   if (self->priv->access_control_param != NULL)
     {
       g_simple_async_report_error_in_idle (G_OBJECT (self), callback,
-          user_data, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+          user_data, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "Can't provide already provided transfer");
 
       return;
@@ -1459,7 +1461,7 @@ tp_file_transfer_channel_provide_file_async (TpFileTransferChannel *self,
       && self->priv->state != TP_FILE_TRANSFER_STATE_PENDING)
     {
       g_simple_async_report_error_in_idle (G_OBJECT (self), callback,
-          user_data, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+          user_data, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "Can't provide a transfer that isn't pending or accepted");
 
       return;
@@ -1468,7 +1470,7 @@ tp_file_transfer_channel_provide_file_async (TpFileTransferChannel *self,
   if (!tp_channel_get_requested (TP_CHANNEL (self)))
     {
       g_simple_async_report_error_in_idle (G_OBJECT (self), callback,
-          user_data, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+          user_data, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "Can't provide incoming transfer");
 
       return;
