@@ -312,7 +312,8 @@ tp_simple_client_factory_constructed (GObject *object)
 {
   TpSimpleClientFactory *self = (TpSimpleClientFactory *) object;
 
-  g_assert (TP_IS_DBUS_DAEMON (self->priv->dbus));
+  if (self->priv->dbus == NULL)
+    self->priv->dbus = tp_dbus_daemon_dup (NULL);
 
   G_OBJECT_CLASS (tp_simple_client_factory_parent_class)->constructed (object);
 }
@@ -398,9 +399,10 @@ tp_simple_client_factory_class_init (TpSimpleClientFactoryClass *klass)
 
 /**
  * tp_simple_client_factory_new:
- * @dbus: a #TpDBusDaemon
+ * @dbus: (allow-none): a #TpDBusDaemon, or %NULL
  *
- * Creates a new #TpSimpleClientFactory instance.
+ * Creates a new #TpSimpleClientFactory instance. If @dbus is %NULL,
+ * tp_dbus_daemon_dup() will be used.
  *
  * Returns: a new #TpSimpleClientFactory
  *
@@ -861,6 +863,177 @@ tp_simple_client_factory_ensure_contact (TpSimpleClientFactory *self,
   return contact;
 }
 
+static void
+upgrade_contacts_cb (GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  TpConnection *connection = (TpConnection *) source;
+  GSimpleAsyncResult *my_result = user_data;
+  GPtrArray *contacts;
+  GError *error = NULL;
+
+  if (!tp_connection_upgrade_contacts_finish (connection, result,
+          &contacts, &error))
+    {
+      g_simple_async_result_take_error (my_result, error);
+    }
+  else
+    {
+      g_simple_async_result_set_op_res_gpointer (my_result, contacts,
+          (GDestroyNotify) g_ptr_array_unref);
+    }
+
+  g_simple_async_result_complete (my_result);
+  g_object_unref (my_result);
+}
+
+
+/**
+ * tp_simple_client_factory_upgrade_contacts_async:
+ * @self: a #TpSimpleClientFactory object
+ * @connection: a #TpConnection
+ * @n_contacts: The number of contacts in @contacts (must be at least 1)
+ * @contacts: (array length=n_contacts): An array of #TpContact objects
+ *  associated with @self
+ * @callback: a callback to call when the operation finishes
+ * @user_data: data to pass to @callback
+ *
+ * Same as tp_connection_upgrade_contacts_async(), but prepare contacts with all
+ * features previously passed to
+ * tp_simple_client_factory_add_contact_features().
+ *
+ * Since: 0.19.1
+ */
+void
+tp_simple_client_factory_upgrade_contacts_async (
+    TpSimpleClientFactory *self,
+    TpConnection *connection,
+    guint n_contacts,
+    TpContact * const *contacts,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GSimpleAsyncResult *result;
+  GArray *features;
+
+  result = g_simple_async_result_new ((GObject *) self, callback, user_data,
+      tp_simple_client_factory_upgrade_contacts_async);
+
+  features = tp_simple_client_factory_dup_contact_features (self, connection);
+  tp_connection_upgrade_contacts_async (connection, n_contacts, contacts,
+      features->len, (TpContactFeature *) features->data,
+      upgrade_contacts_cb, result);
+  g_array_unref (features);
+}
+
+/**
+ * tp_simple_client_factory_upgrade_contacts_finish:
+ * @self: a #TpSimpleClientFactory
+ * @result: a #GAsyncResult
+ * @contacts: (element-type TelepathyGLib.Contact) (transfer container) (out) (allow-none):
+ *  a location to set a #GPtrArray of upgraded #TpContact, or %NULL.
+ * @error: a #GError to fill
+ *
+ * Finishes tp_simple_client_factory_upgrade_contacts_async()
+ *
+ * Returns: %TRUE on success, %FALSE otherwise.
+ * Since: 0.19.1
+ */
+gboolean
+tp_simple_client_factory_upgrade_contacts_finish (
+    TpSimpleClientFactory *self,
+    GAsyncResult *result,
+    GPtrArray **contacts,
+    GError **error)
+{
+  _tp_implement_finish_copy_pointer (self,
+      tp_simple_client_factory_upgrade_contacts_async,
+      g_ptr_array_ref, contacts);
+}
+
+static void
+dup_contact_by_id_cb (GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  TpConnection *connection = (TpConnection *) source;
+  GSimpleAsyncResult *my_result = user_data;
+  TpContact *contact;
+  GError *error = NULL;
+
+  contact = tp_connection_dup_contact_by_id_finish (connection, result, &error);
+  if (contact == NULL)
+    {
+      g_simple_async_result_take_error (my_result, error);
+    }
+  else
+    {
+      g_simple_async_result_set_op_res_gpointer (my_result, contact,
+          g_object_unref);
+    }
+
+  g_simple_async_result_complete (my_result);
+  g_object_unref (my_result);
+}
+
+
+/**
+ * tp_simple_client_factory_ensure_contact_by_id_async:
+ * @self: a #TpSimpleClientFactory object
+ * @connection: a #TpConnection
+ * @identifier: a string representing the contact's identifier
+ * @callback: a callback to call when the operation finishes
+ * @user_data: data to pass to @callback
+ *
+ * Same as tp_connection_dup_contact_by_id_async(), but prepare the
+ * contact with all features previously passed to
+ * tp_simple_client_factory_add_contact_features().
+ *
+ * Since: 0.19.1
+ */
+void
+tp_simple_client_factory_ensure_contact_by_id_async (
+    TpSimpleClientFactory *self,
+    TpConnection *connection,
+    const gchar *identifier,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GSimpleAsyncResult *result;
+  GArray *features;
+
+  result = g_simple_async_result_new ((GObject *) self, callback, user_data,
+      tp_simple_client_factory_ensure_contact_by_id_async);
+
+  features = tp_simple_client_factory_dup_contact_features (self, connection);
+  tp_connection_dup_contact_by_id_async (connection, identifier,
+      features->len, (TpContactFeature *) features->data,
+      dup_contact_by_id_cb, result);
+  g_array_unref (features);
+}
+
+/**
+ * tp_simple_client_factory_ensure_contact_by_id_finish:
+ * @self: a #TpSimpleClientFactory
+ * @result: a #GAsyncResult
+ * @error: a #GError to fill
+ *
+ * Finishes tp_simple_client_factory_ensure_contact_by_id_async()
+ *
+ * Returns: (transfer full): a #TpContact or %NULL on error.
+ * Since: 0.19.1
+ */
+TpContact *
+tp_simple_client_factory_ensure_contact_by_id_finish (
+    TpSimpleClientFactory *self,
+    GAsyncResult *result,
+    GError **error)
+{
+  _tp_implement_finish_return_copy_pointer (self,
+      tp_simple_client_factory_ensure_contact_by_id_async, g_object_ref);
+}
+
 /**
  * tp_simple_client_factory_dup_contact_features:
  * @self: a #TpSimpleClientFactory object
