@@ -25,6 +25,7 @@
 #include <telepathy-glib/util.h>
 
 #include "textchan-null.h"
+#include "room-list-chan.h"
 #include "util.h"
 
 static void conn_iface_init (TpSvcConnectionClass *);
@@ -59,7 +60,8 @@ struct _TpTestsSimpleConnectionPrivate
   gboolean break_fastpath_props;
 
   /* TpHandle => reffed TpTestsTextChannelNull */
-  GHashTable *channels;
+  GHashTable *text_channels;
+  TpTestsRoomListChan *room_list_chan;
 
   GError *get_self_handle_error /* initially NULL */ ;
 };
@@ -70,7 +72,7 @@ tp_tests_simple_connection_init (TpTestsSimpleConnection *self)
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
       TP_TESTS_TYPE_SIMPLE_CONNECTION, TpTestsSimpleConnectionPrivate);
 
-  self->priv->channels = g_hash_table_new_full (NULL, NULL, NULL,
+  self->priv->text_channels = g_hash_table_new_full (NULL, NULL, NULL,
       (GDestroyNotify) g_object_unref);
 }
 
@@ -97,12 +99,8 @@ get_property (GObject *object,
         }
       else
         {
-          guint32 status = TP_BASE_CONNECTION (self)->status;
-
-          if (status == TP_INTERNAL_CONNECTION_STATUS_NEW)
-            g_value_set_uint (value, TP_CONNECTION_STATUS_DISCONNECTED);
-          else
-            g_value_set_uint (value, status);
+          g_value_set_uint (value,
+              tp_base_connection_get_status (TP_BASE_CONNECTION (self)));
         }
       break;
     default:
@@ -136,7 +134,8 @@ dispose (GObject *object)
 {
   TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (object);
 
-  g_hash_table_unref (self->priv->channels);
+  g_hash_table_unref (self->priv->text_channels);
+  g_clear_object (&self->priv->room_list_chan);
 
   G_OBJECT_CLASS (tp_tests_simple_connection_parent_class)->dispose (object);
 }
@@ -178,14 +177,14 @@ tp_tests_simple_normalize_contact (TpHandleRepoIface *repo,
 {
   if (id[0] == '\0')
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_HANDLE,
+      g_set_error (error, TP_ERROR, TP_ERROR_INVALID_HANDLE,
           "ID must not be empty");
       return NULL;
     }
 
   if (strchr (id, ' ') != NULL)
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_HANDLE,
+      g_set_error (error, TP_ERROR, TP_ERROR_INVALID_HANDLE,
           "ID must not contain spaces");
       return NULL;
     }
@@ -195,7 +194,7 @@ tp_tests_simple_normalize_contact (TpHandleRepoIface *repo,
 
 static void
 create_handle_repos (TpBaseConnection *conn,
-                     TpHandleRepoIface *repos[NUM_TP_HANDLE_TYPES])
+                     TpHandleRepoIface *repos[TP_NUM_HANDLE_TYPES])
 {
   repos[TP_HANDLE_TYPE_CONTACT] = tp_dynamic_handle_repo_new
       (TP_HANDLE_TYPE_CONTACT, tp_tests_simple_normalize_contact, NULL);
@@ -224,11 +223,13 @@ pretend_connected (gpointer data)
   TpBaseConnection *conn = (TpBaseConnection *) self;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_CONTACT);
+  TpHandle self_handle;
 
-  conn->self_handle = tp_handle_ensure (contact_repo, self->priv->account,
+  self_handle = tp_handle_ensure (contact_repo, self->priv->account,
       NULL, NULL);
+  tp_base_connection_set_self_handle (conn, self_handle);
 
-  if (conn->status == TP_CONNECTION_STATUS_CONNECTING)
+  if (tp_base_connection_get_status (conn) == TP_CONNECTION_STATUS_CONNECTING)
     {
       tp_base_connection_change_status (conn, TP_CONNECTION_STATUS_CONNECTED,
           TP_CONNECTION_STATUS_REASON_REQUESTED);
@@ -262,7 +263,8 @@ pretend_disconnected (gpointer data)
   TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (data);
 
   /* We are disconnected, all our channels are invalidated */
-  g_hash_table_remove_all (self->priv->channels);
+  g_hash_table_remove_all (self->priv->text_channels);
+  g_clear_object (&self->priv->room_list_chan);
 
   tp_base_connection_finish_shutdown (TP_BASE_CONNECTION (data));
   self->priv->disconnect_source = 0;
@@ -282,6 +284,19 @@ shut_down (TpBaseConnection *conn)
       conn);
 }
 
+static GPtrArray *
+get_interfaces_always_present (TpBaseConnection *base)
+{
+  GPtrArray *interfaces;
+
+  interfaces = TP_BASE_CONNECTION_CLASS (
+      tp_tests_simple_connection_parent_class)->get_interfaces_always_present (base);
+
+  g_ptr_array_add (interfaces, TP_IFACE_CONNECTION_INTERFACE_REQUESTS);
+
+  return interfaces;
+}
+
 static void
 tp_tests_simple_connection_class_init (TpTestsSimpleConnectionClass *klass)
 {
@@ -289,8 +304,6 @@ tp_tests_simple_connection_class_init (TpTestsSimpleConnectionClass *klass)
       (TpBaseConnectionClass *) klass;
   GObjectClass *object_class = (GObjectClass *) klass;
   GParamSpec *param_spec;
-  static const gchar *interfaces_always_present[] = {
-      TP_IFACE_CONNECTION_INTERFACE_REQUESTS, NULL };
 
   object_class->get_property = get_property;
   object_class->set_property = set_property;
@@ -304,7 +317,7 @@ tp_tests_simple_connection_class_init (TpTestsSimpleConnectionClass *klass)
   base_class->start_connecting = start_connecting;
   base_class->shut_down = shut_down;
 
-  base_class->interfaces_always_present = interfaces_always_present;
+  base_class->get_interfaces_always_present = get_interfaces_always_present;
 
   param_spec = g_param_spec_string ("account", "Account name",
       "The username of this user", NULL,
@@ -346,7 +359,6 @@ tp_tests_simple_connection_set_identifier (TpTestsSimpleConnection *self,
   g_return_if_fail (handle != 0);
 
   tp_base_connection_set_self_handle (conn, handle);
-  tp_handle_unref (contact_repo, handle);
 }
 
 TpTestsSimpleConnection *
@@ -379,7 +391,8 @@ tp_tests_simple_connection_ensure_text_chan (TpTestsSimpleConnection *self,
 
   handle = tp_handle_ensure (contact_repo, target_id, NULL, NULL);
 
-  chan = g_hash_table_lookup (self->priv->channels, GUINT_TO_POINTER (handle));
+  chan = g_hash_table_lookup (self->priv->text_channels,
+      GUINT_TO_POINTER (handle));
   if (chan != NULL)
     {
       /* Channel already exist, reuse it */
@@ -387,8 +400,8 @@ tp_tests_simple_connection_ensure_text_chan (TpTestsSimpleConnection *self,
     }
   else
     {
-      chan_path = g_strdup_printf ("%s/Channel%u", base_conn->object_path,
-          count++);
+      chan_path = g_strdup_printf ("%s/Channel%u",
+          tp_base_connection_get_object_path (base_conn), count++);
 
        chan = TP_TESTS_TEXT_CHANNEL_NULL (
           tp_tests_object_new_static_class (
@@ -398,18 +411,61 @@ tp_tests_simple_connection_ensure_text_chan (TpTestsSimpleConnection *self,
             "handle", handle,
             NULL));
 
-      g_hash_table_insert (self->priv->channels, GUINT_TO_POINTER (handle),
+      g_hash_table_insert (self->priv->text_channels, GUINT_TO_POINTER (handle),
           chan);
     }
 
-  tp_handle_unref (contact_repo, handle);
-
   if (props != NULL)
     *props = tp_tests_text_channel_get_props (chan);
 
   return chan_path;
 }
 
+static void
+room_list_chan_closed_cb (TpBaseChannel *channel,
+    TpTestsSimpleConnection *self)
+{
+  g_clear_object (&self->priv->room_list_chan);
+}
+
+gchar *
+tp_tests_simple_connection_ensure_room_list_chan (TpTestsSimpleConnection *self,
+    const gchar *server,
+    GHashTable **props)
+{
+  gchar *chan_path;
+  TpBaseConnection *base_conn = (TpBaseConnection *) self;
+
+  if (self->priv->room_list_chan != NULL)
+    {
+      /* Channel already exist, reuse it */
+      g_object_get (self->priv->room_list_chan,
+          "object-path", &chan_path, NULL);
+    }
+  else
+    {
+      chan_path = g_strdup_printf ("%s/RoomListChannel",
+          tp_base_connection_get_object_path (base_conn));
+
+      self->priv->room_list_chan = TP_TESTS_ROOM_LIST_CHAN (
+          tp_tests_object_new_static_class (
+            TP_TESTS_TYPE_ROOM_LIST_CHAN,
+            "connection", self,
+            "object-path", chan_path,
+            "server", server ? server : "",
+            NULL));
+
+      g_signal_connect (self->priv->room_list_chan, "closed",
+          G_CALLBACK (room_list_chan_closed_cb), self);
+    }
+
+  if (props != NULL)
+    g_object_get (self->priv->room_list_chan,
+        "channel-properties", props, NULL);
+
+  return chan_path;
+}
+
 void
 tp_tests_simple_connection_set_get_self_handle_error (
     TpTestsSimpleConnection *self,
@@ -438,7 +494,8 @@ get_self_handle (TpSvcConnection *iface,
       return;
     }
 
-  tp_svc_connection_return_from_get_self_handle (context, base->self_handle);
+  tp_svc_connection_return_from_get_self_handle (context,
+      tp_base_connection_get_self_handle (base));
   g_signal_emit (self, signals[SIGNAL_GOT_SELF_HANDLE], 0);
 }
 
